package com.ads.business.service.impl;

import com.ads.business.service.WechatService;
import com.ads.business.service.data.aftersale.AftersaleAddReq;
import com.ads.business.service.data.aftersale.AftersaleRes;
import com.ads.business.service.data.brand.BrandAuditPost;
import com.ads.business.service.data.brand.BrandAuditRes;
import com.ads.business.service.data.category.CatAuditPost;
import com.ads.business.service.data.category.CatAuditRes;
import com.ads.business.service.data.delivery.DeliverySendReq;
import com.ads.business.service.data.delivery.DeliverySendRes;
import com.ads.business.service.data.order.OrderGetReq;
import com.ads.business.service.data.order.OrderInfoRes;
import com.ads.business.service.data.order.OrderPayReq;
import com.ads.business.service.data.order.OrderPayRes;
import com.ads.business.service.data.spu.Spu;
import com.ads.business.service.data.spu.SpuData;
import com.ads.business.service.data.spu.SpuDetail;
import com.ads.common.annotation.TokenValidator;
import com.ads.common.constant.CodeConstant;
import com.ads.common.constant.UrlConstant;
import com.ads.common.enums.ErrCodeEnum;
import com.ads.common.exception.WechatApiErrException;
import com.ads.common.handler.WeChatHandler;
import org.apache.http.conn.ConnectTimeoutException;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.EnableRetry;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.net.SocketTimeoutException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author huzhihao
 * @date 2021-12-21 16:00
 **/
@Service
@EnableRetry
public class WechatServiceImpl implements WechatService {


    /**
     * [上传图片]
     *
     * @param file
     * @return java.lang.String
     * @author huzhihao
     * @date 2021/12/26 9:53
     */
    @Override
    public String upload(MultipartFile file) {
        return WeChatHandler.httpUpload(file);
    }

    /**
     * [添加商品]
     *
     * @param param
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/26 10:12
     */
    @TokenValidator
    @Override
    public Spu spuAdd(Spu param) {
        SpuData ret = (SpuData) WeChatHandler.httpPostMethod(UrlConstant.SPU_ADD, param, SpuData.class);
        return ret.getData();
    }

    /**
     * [更新商品]
     *
     * @param param
     * @return com.ads.business.service.data.spu.Spu
     * @author huzhihao
     * @date 2021/12/26 14:11
     */
    @TokenValidator
    @Override
    public Spu spuUpdate(Object param) {
        SpuData ret = (SpuData) WeChatHandler.httpPostMethod(UrlConstant.SPU_UPDATE, param, SpuData.class);
        return ret.getData();
    }

    /**
     * [获取商品]
     *
     * @param outProductId
     * @return com.ads.business.service.data.spu.Spu
     * @author huzhihao
     * @date 2021/12/26 14:12
     */
    @TokenValidator
    @Override
    public Spu spuGet(String outProductId) {
        Map param = new HashMap();
        param.put(CodeConstant.WECHAT.OUT_PRODUCT_ID, outProductId);
        param.put(CodeConstant.WECHAT.NEED_EDIT_SPU, 1);
        SpuDetail ret = (SpuDetail) WeChatHandler.httpPostMethod(UrlConstant.SPU_GET, param, SpuDetail.class);
        return ret.getSpu();
    }

    /**
     * [删除商品]
     *
     * @param outProductId
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/26 14:12
     */
    @TokenValidator
    @Override
    public Boolean spuDel(String outProductId) {
        Map param = new HashMap();
        param.put(CodeConstant.WECHAT.OUT_PRODUCT_ID, outProductId);
        WeChatHandler.httpPostMethod(UrlConstant.SPU_DEL, param, SpuDetail.class);
        return true;
    }

    /**
     * [上架商品]
     *
     * @param outProductId
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/26 14:12
     */
    @TokenValidator
    @Override
    public Boolean spuListing(String outProductId) {
        Map param = new HashMap();
        param.put(CodeConstant.WECHAT.OUT_PRODUCT_ID, outProductId);
        WeChatHandler.httpPostMethod(UrlConstant.SPU_LISTING, param, SpuDetail.class);
        return true;
    }

    /**
     * [下架商品]
     *
     * @param outProductId
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/26 14:12
     */
    @TokenValidator
    @Override
    public Boolean spuDelisting(String outProductId) {
        Map param = new HashMap();
        param.put(CodeConstant.WECHAT.OUT_PRODUCT_ID, outProductId);
        WeChatHandler.httpPostMethod(UrlConstant.SPU_DELISTING, param, SpuDetail.class);
        return true;
    }


    /**
     * [订单同步]
     *
     * @param param
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/28 9:06
     */
    @Retryable(
            value = {
                    WechatApiErrException.class,
                    SocketTimeoutException.class,
                    ConnectTimeoutException.class
            },
            maxAttempts = 3,
            backoff = @Backoff(delay = 1000, multiplier = 2)
    )
    @TokenValidator
    @Override
    public Boolean orderPay(OrderPayReq param) {
        OrderPayRes o = (OrderPayRes) WeChatHandler.httpPostMethod(UrlConstant.ORDER_PAY, param, OrderPayRes.class);
        if (ErrCodeEnum.OK.getCode().equals(o.getErrcode().toString())) {
            return true;
        }
        return false;
    }

    /**
     * [获取订单详情]
     *
     * @param param
     * @return com.ads.business.service.data.order.OrderInfoRes
     * @author huzhihao
     * @date 2021/12/29 14:17
     */
    @TokenValidator
    @Override
    public OrderInfoRes orderGet(OrderGetReq param) {
        return (OrderInfoRes) WeChatHandler.httpPostMethod(UrlConstant.ORDER_GET, param, OrderInfoRes.class);
    }

    /**
     * [物流发货]
     *
     * @param param
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2021/12/28 9:06
     */
    @Retryable(
            value = {
                    WechatApiErrException.class,
                    SocketTimeoutException.class,
                    ConnectTimeoutException.class
            },
            maxAttempts = 3,
            backoff = @Backoff(delay = 1000, multiplier = 2)
    )
    @TokenValidator
    @Override
    public Boolean deliverySend(DeliverySendReq param) {
        DeliverySendRes o = (DeliverySendRes) WeChatHandler.httpPostMethod(UrlConstant.DELIVERY_SEND, param, DeliverySendRes.class);
        if (ErrCodeEnum.OK.getCode().equals(o.getErrcode().toString())) {
            return true;
        }
        return false;
    }

    /**
     * [品牌提交]
     *
     * @param param
     * @return java.lang.String
     * @author huzhihao
     * @date 2021/12/30 19:47
     */
    @TokenValidator
    @Override
    public String auditBrand(BrandAuditPost param) {
        BrandAuditRes res = (BrandAuditRes) WeChatHandler.httpPostMethod(UrlConstant.AUDIT_BRAND, param, BrandAuditRes.class);
        return res.getAudit_id();
    }

    /**
     * [类目提交]
     *
     * @param param
     * @return java.lang.String
     * @author huzhihao
     * @date 2021/12/30 19:47
     */
    @TokenValidator
    @Override
    public String auditCategory(CatAuditPost param) {
        CatAuditRes res = (CatAuditRes) WeChatHandler.httpPostMethod(UrlConstant.AUDIT_CATEGORY, param, CatAuditRes.class);
        return res.getAudit_id();
    }

    /**
     * [创建售后]
     *
     * @param param
     * @return java.lang.Boolean
     * @author huzhihao
     * @date 2022/1/6 15:11
     */
    @Retryable(
            value = {
                    WechatApiErrException.class,
                    SocketTimeoutException.class,
                    ConnectTimeoutException.class
            },
            maxAttempts = 3,
            backoff = @Backoff(delay = 1000, multiplier = 2)
    )
    @TokenValidator
    @Override
    public Boolean aftersaleAdd(AftersaleAddReq param) {
        AftersaleRes o = (AftersaleRes) WeChatHandler.httpPostMethod(UrlConstant.AFTERSALE_SEND, param, AftersaleRes.class);
        if (ErrCodeEnum.OK.getCode().equals(o.getErrcode().toString())) {
            return true;
        }
        return false;
    }
}